MODULE carbon_cycle_mod

!=======================================================================
!
!   Authors: Patricia Cadule and Laurent Fairhead
!            base sur un travail anterieur mene par Patricia Cadule et Josefine
!            Ghattas
!   -------
!
!  Purpose and description:
!  -----------------------
! Controle module for the carbon CO2 tracers :
!   - Identification
!   - Get concentrations comming from coupled model or read from file to tracers
!   - Calculate new RCO2 for radiation scheme
!   - Calculate new carbon flux for sending to coupled models (PISCES and ORCHIDEE)
!
! Module permettant de mettre a jour les champs (puits et sources) pour le transport de CO2 en
! online (IPSL-CM et LMDZOR) et offline (lecture de carte)
!   
! Le cas online/offline est defini par le flag carbon_cycle_cpl (y/n)
! Le transport du traceur CO2 est defini par le flag carbon_cycle_tr (y/n)
! la provenance des champs (termes de puits) est denini par le flag  level_coupling_esm

! level_coupling_esm : level of coupling of the biogeochemical fields between
! LMDZ, ORCHIDEE and NEMO
! Definitions of level_coupling_esm in physiq.def
! level_coupling_esm = 0  ! No field exchange between LMDZ and ORCHIDEE models
!                         ! No field exchange between LMDZ and NEMO
! level_coupling_esm = 1  ! Field exchange between LMDZ and ORCHIDEE models
!                         ! No field exchange between LMDZ and NEMO models
! level_coupling_esm = 2  ! No field exchange between LMDZ and ORCHIDEE models
!                         ! Field exchange between LMDZ and NEMO models
! level_coupling_esm = 3  ! Field exchange between LMDZ and ORCHIDEE models
!                         ! Field exchange between LMDZ and NEMO models
!
! si carbon_cycle_cpl=y alors les termes de puits et sources sont mis a jour 
! en focntion de la valeur de level_coupling_esm
! dans tous les cas  (si carbon_cycle_cpl=y) les valeurs des emissions anthropiques de CO2 (fossil
! fuel+cement) sont lues,
! - soit via des cartes 2D mensuelles (fichiers netcdf contenant 12 valeurs par
! annee): fCO2_fco2fos
! - soit via des fichier txt contenant une valeur globale mensuelle/annuelle: fCO2_fco2fos_1D
!
! Ce module permet de transport des champs de carbone dans LMDz

! Declaration des differents champs 
! LAND: fCO2_nep, fCO2_fLuc, fCO2_fFire,
! OCEAN: fCO2_fgco2
! FOS:fCO2_fco2fos

  IMPLICIT NONE
  SAVE
  PRIVATE
  PUBLIC :: carbon_cycle_init, carbon_cycle

! Variables read from parmeter file physiq.def
  LOGICAL, PUBLIC :: carbon_cycle_tr        ! 3D transport of CO2 in the atmosphere, parameter read in conf_phys
!$OMP THREADPRIVATE(carbon_cycle_tr)
  LOGICAL, PUBLIC :: carbon_cycle_cpl       ! Coupling of CO2 fluxes between LMDZ/ORCHIDEE and LMDZ/OCEAN(PISCES) 
!$OMP THREADPRIVATE(carbon_cycle_cpl)
  INTEGER, SAVE, PUBLIC :: level_coupling_esm
!$OMP THREADPRIVATE(level_coupling_esm)

  LOGICAL :: carbon_cycle_emis_comp_omp=.FALSE.
  LOGICAL :: carbon_cycle_emis_comp=.FALSE. ! Calculation of emission compatible
!$OMP THREADPRIVATE(carbon_cycle_emis_comp)

  LOGICAL :: RCO2_inter_omp
  LOGICAL :: RCO2_inter  ! RCO2 interactive : if true calculate new value RCO2 for the radiation scheme
!$OMP THREADPRIVATE(RCO2_inter)

! Scalar values when no transport, from physiq.def
  REAL :: fCO2_fco2fos_1D_omp
!$OMP THREADPRIVATE(fCO2_fco2fos_1D_omp)
  REAL :: fCO2_fco2fos_1D  ! carbon_cycle_fco2fos dans physiq.def
!$OMP THREADPRIVATE(fCO2_fco2fos_1D)
  REAL :: emis_land_s ! not yet implemented
!$OMP THREADPRIVATE(emis_land_s)

  REAL :: airetot     ! Total area of the earth surface
!$OMP THREADPRIVATE(airetot)

  INTEGER :: ntr_co2  ! Number of fields concerning the carbon cycle
!$OMP THREADPRIVATE(ntr_co2)

! fCO2_fgco2 : carbon flux from ocean cumulated between 2 ATM-OCN coupling timestep [gC/m2/Dt]. 
! fCO2_fgco2 :  flux de carbone venant de l ocean. Ce flux est cumule entre
! deux pas de temps du couplage ATM-OCN [gC/m2/Dt]. 
! Allocation and initalization done in cpl_mod

! OCEAN
! fkux echange a la frequence de couplage entre NEMO et LMDZ
  REAL, DIMENSION(:), ALLOCATABLE, PUBLIC :: fCO2_fgco2 
!$OMP THREADPRIVATE(fCO2_fgco2)

! LAND
  REAL, DIMENSION(:), ALLOCATABLE :: fCO2_nbp        ! carbon flux from land at each time step  [kgC/m2/s]
!$OMP THREADPRIVATE(fCO2_nbp)
  REAL, DIMENSION(:), ALLOCATABLE :: fCO2_nep        ! carbon flux from land at each time step  [kgC/m2/s]
!$OMP THREADPRIVATE(fCO2_nep)
  REAL, DIMENSION(:), ALLOCATABLE :: fCO2_fLuc     ! Emission from land use change  [gC/m2/s]
!$OMP THREADPRIVATE(fCO2_fLuc)
  REAL, DIMENSION(:), ALLOCATABLE :: fCO2_fFire
!$OMP THREADPRIVATE(fCO2_fFire)

! SOURCES: FOSSIL FUEL emissions
  REAL, DIMENSION(:), ALLOCATABLE :: fCO2_fco2fos 
!$OMP THREADPRIVATE(fCO2_fco2fos)

  REAL, DIMENSION(:,:), ALLOCATABLE :: dtr_add       ! Tracer concentration to be injected 
!$OMP THREADPRIVATE(dtr_add)
  REAL, DIMENSION(:), ALLOCATABLE :: dtr_add_sum       ! Tracer concentration to be injected
!$OMP THREADPRIVATE(dtr_add_sum)

! Following 4 fields will be allocated and initialized in surf_land_orchidee
  REAL, DIMENSION(:), ALLOCATABLE, PUBLIC :: fCO2_nbp_inst  ! flux CO2 from land at one time step
!$OMP THREADPRIVATE(fCO2_nbp_inst)
  REAL, DIMENSION(:), ALLOCATABLE, PUBLIC :: fCO2_nep_inst  ! flux CO2 from land at one time step
!$OMP THREADPRIVATE(fCO2_nep_inst)
  REAL, DIMENSION(:), ALLOCATABLE, PUBLIC :: fCO2_fLuc_inst    ! Emission from land use change at one time step
!$OMP THREADPRIVATE(fCO2_fLuc_inst)
  REAL, DIMENSION(:), ALLOCATABLE, PUBLIC :: fCO2_fFire_inst  ! flux CO2 from land at one time step
!$OMP THREADPRIVATE(fCO2_fFire_inst)

! Calculated co2 field to be send to the ocean model (NEMO) via the coupler and to the land surface model (ORCHIDEE) 
  REAL, DIMENSION(:), ALLOCATABLE, PUBLIC :: co2_send ! Field allocated in phyetat0
!$OMP THREADPRIVATE(co2_send)


  TYPE, PUBLIC ::   co2_fields_type
     CHARACTER(len = 20) :: name       ! Fields 
     INTEGER            :: id         ! Index in total tracer list, tr_seri
     CHARACTER(len=30)  :: file       ! File name
     LOGICAL            :: cpl        ! True if this tracers is coupled from ORCHIDEE or PISCES. 
                                      ! False if read from file.
     INTEGER            :: updatefreq ! Frequence to inject in second
     INTEGER            :: readstep   ! Actual time step to read in file
     LOGICAL            :: updatenow  ! True if this tracer should be updated this time step
  END TYPE co2_fields_type
  INTEGER,PARAMETER :: maxco2fields=5  ! Maximum number of different CO2 fluxes
  TYPE(co2_fields_type), DIMENSION(maxco2fields) :: co2fields
!$OMP THREADPRIVATE(co2fields)


  TYPE, PUBLIC ::   co2_trac_type
     CHARACTER(len = 8) :: name       ! Tracer name in tracer.def
     INTEGER            :: id         ! Index in total tracer list, tr_seri
     CHARACTER(len=30)  :: file       ! File name
     LOGICAL            :: cpl        ! True if this tracers is coupled from ORCHIDEE or PISCES.
                                      ! False if read from file.
     INTEGER            :: updatefreq ! Frequence to inject in second
     INTEGER            :: readstep   ! Actual time step to read in file
     LOGICAL            :: updatenow  ! True if this tracer should be updated this time step
  END TYPE co2_trac_type
  INTEGER,PARAMETER :: maxco2trac=5  ! Maximum number of different CO2 fluxes
  TYPE(co2_trac_type), DIMENSION(maxco2trac) :: co2trac



CONTAINS
  
  SUBROUTINE carbon_cycle_init(tr_seri, pdtphys, aerosol, radio)
! This subroutine is called from traclmdz_init, only at first timestep.
! - Read controle parameters from .def input file
! - Search for carbon tracers and set default values
! - Allocate variables
! - Test for compatibility

    USE dimphy
    USE geometry_mod, ONLY : cell_area 
    USE mod_phys_lmdz_transfert_para
    USE infotrac_phy, ONLY: nbtr, nqo, niadv, tname
    USE IOIPSL
    USE surface_data, ONLY : ok_veget, type_ocean
    USE phys_cal_mod, ONLY : mth_len
    USE print_control_mod, ONLY: lunout
    USE infocfields_mod

    IMPLICIT NONE
    INCLUDE "clesphys.h"
 
! Input argument
    REAL,DIMENSION(klon,klev,nbtr),INTENT(IN) :: tr_seri ! Concentration Traceur [U/KgA]  
    REAL,INTENT(IN)                           :: pdtphys ! length of time step in physiq (sec)

! InOutput arguments
    LOGICAL,DIMENSION(nbtr), INTENT(INOUT) :: aerosol
    LOGICAL,DIMENSION(nbtr), INTENT(INOUT) :: radio

! Local variables
    INTEGER               :: ierr, it, iiq, itc
    INTEGER               :: teststop



! 1) Read controle parameters from .def input file
! ------------------------------------------------
    ! Read fosil fuel value if no transport
    IF (.NOT. carbon_cycle_tr) THEN
!$OMP MASTER
       fCO2_fco2fos_1D_omp = 0.
       CALL getin ('carbon_cycle_fco2fos_1D',fCO2_fco2fos_1D_omp)
!$OMP END MASTER
!$OMP BARRIER
       fCO2_fco2fos_1D=fCO2_fco2fos_1D_omp
       CALL bcast(fCO2_fco2fos_1D)
       WRITE(lunout,*) 'carbon_cycle_fco2fos_1D = ', fCO2_fco2fos_1D
    END IF


    ! Read parmeter for calculation compatible emission
    IF (.NOT. carbon_cycle_tr) THEN
!$OMP MASTER
       carbon_cycle_emis_comp_omp=.FALSE.
       CALL getin('carbon_cycle_emis_comp',carbon_cycle_emis_comp_omp)
!$OMP END MASTER
!$OMP BARRIER
       carbon_cycle_emis_comp=carbon_cycle_emis_comp_omp
       WRITE(lunout,*) 'carbon_cycle_emis_comp = ',carbon_cycle_emis_comp
       IF (carbon_cycle_emis_comp) THEN
          CALL abort_physic('carbon_cycle_init', 'carbon_cycle_emis_comp option not yet implemented!!',1)
       END IF
    END IF

    ! Read parameter for interactive calculation of the CO2 value for the radiation scheme
!$OMP MASTER
    RCO2_inter_omp=.FALSE.
    CALL getin('RCO2_inter',RCO2_inter_omp)
!$OMP END MASTER
!$OMP BARRIER
    RCO2_inter=RCO2_inter_omp
    WRITE(lunout,*) 'RCO2_inter = ', RCO2_inter
    IF (RCO2_inter) THEN
       WRITE(lunout,*) 'RCO2 will be recalculated once a day'
       WRITE(lunout,*) 'RCO2 initial = ', RCO2
    END IF


! 2) Search for carbon fields and set default values
! ---------------------------------------------------
    itc=0
    DO it=1,nbcf_in  ! bien remplir le fichier pour avoir un champ fos et un champs ocean

!fCO2_nbp, fCO2_nep, fCO2_fLuc, fCO2_fFire, fCO2_fco2fos, fCO2_fgco2       

       SELECT CASE(cfname_in(it))
       !SELECT CASE(co2fields(it)%name)
       CASE("fCO2_fgco2")
          itc = itc + 1
          co2fields(it)%name=TRIM(cfname_in(it))
          co2fields(it)%id=it
          co2fields(it)%file='flx_fgco2_ocean.nc'
          IF (carbon_cycle_cpl .AND. (type_ocean=='couple') .AND. (level_coupling_esm.GE.2))  THEN 
             WRITE(lunout,*) 'carbon_cycle_mod (ESM) --- fCO2_fgco2'
             co2fields(it)%cpl=.TRUE.
! >> PC
! on va injecter a tous les pas de temps de couplage (2700x2)
! recuperer proprement la frequence de couplage
! << PC
!!!! METTRE la vraie frequence de couplage !!! PAS en dur
             co2fields(it)%updatefreq = 5400 !  OASIS/NEMO-PISCES
! << PC
          ELSE IF (carbon_cycle_cpl .AND.  (level_coupling_esm.LE.2))  THEN 
             co2fields(it)%cpl=.FALSE.
             co2fields(it)%updatefreq = 86400*mth_len ! Once a month
          END IF
       CASE("fCO2_nbp")
          itc = itc + 1
          co2fields(it)%name=TRIM(cfname_in(it))
          co2fields(it)%id=it
          co2fields(it)%file='flx_nbp_land.nc'
          IF (carbon_cycle_cpl .AND. ((level_coupling_esm.EQ.1) .OR. level_coupling_esm.EQ.3)) THEN
             WRITE(lunout,*) 'carbon_cycle_mod (ESM) --- fCO2_nbp'
             co2fields(it)%cpl=.TRUE.
             co2fields(it)%updatefreq = INT(pdtphys) ! Each timestep as the coupling with ORCHIDEE
          ELSE IF (carbon_cycle_cpl .AND. ((level_coupling_esm.EQ.0) .OR.  level_coupling_esm.EQ.2)) THEN
             co2fields(it)%cpl=.FALSE.
!             co2fields(itc)%updatefreq = 10800   ! 10800sec = 3H
             co2fields(it)%updatefreq = 86400*mth_len ! Once a month
          END IF
!!!! Si le test est ok pour nbp dupliquer sur les autres champs LAND
       CASE("fCO2_nep")
          itc = itc + 1
          co2fields(it)%name=TRIM(cfname_in(it))
          co2fields(it)%id=it
          co2fields(it)%file='flx_nep_land.nc'
          IF (carbon_cycle_cpl .AND. ok_veget) THEN 
             WRITE(lunout,*) 'carbon_cycle_mod (ESM) --- fCO2_nep'
             co2fields(it)%cpl=.TRUE.
             co2fields(it)%updatefreq = INT(pdtphys) ! Each timestep as the coupling with ORCHIDEE
          ELSE
             co2fields(it)%cpl=.FALSE.
!             co2fields(itc)%updatefreq = 10800   ! 10800sec = 3H
             co2fields(it)%updatefreq = 86400*mth_len ! Once a month
          END IF
       CASE("fCO2_fLuc")
          itc = itc + 1
          co2fields(it)%name=TRIM(cfname_in(it))
          co2fields(it)%id=it
          co2fields(it)%file='flx_fLuc_land.nc'
          IF (carbon_cycle_cpl .AND. ok_veget) THEN 
             WRITE(lunout,*) 'carbon_cycle_mod (ESM) --- fCO2_nbp'
             co2fields(it)%cpl=.TRUE.
             co2fields(it)%updatefreq = INT(pdtphys) ! Each timestep as the coupling with ORCHIDEE
          ELSE
             co2fields(it)%cpl=.FALSE.
             co2fields(it)%updatefreq = 10800   ! 10800sec = 3H
          END IF
       CASE("fCO2_fco2fos")
          itc = itc + 1
          !co2fields(it)%name='fCO2_fco2fos'
          co2fields(it)%name=TRIM(cfname_in(it))
          co2fields(it)%id=it
          co2fields(it)%file='fco2fos.nc' 
! >> PC
         IF ( carbon_cycle_cpl .AND. carbon_cycle_tr) THEN 
          WRITE(lunout,*) 'carbon_cycle_mod (ESM) --- fCO2_fco2fos'
          co2fields(it)%cpl=.TRUE.
          co2fields(it)%updatefreq = 86400*mth_len ! Once a month
         ELSE IF ( carbon_cycle_cpl .AND. (.NOT. carbon_cycle_tr)) THEN 
          co2fields(it)%cpl=.TRUE.
          co2fields(it)%updatefreq = 86400*mth_len ! Once a month
            WRITE(*,*) "lecture du champs scalaire"
         END IF
! << PC
       CASE("fCO2_fFire")
          itc = itc + 1
          co2fields(it)%name=TRIM(cfname_in(it))
          co2fields(it)%id=it
          co2fields(it)%file='flx_fFire_land.nc'
          co2fields(it)%cpl=.FALSE.       ! This tracer always read from file
          co2fields(it)%updatefreq = 86400*mth_len ! Once a month
       CASE("fCO2")
          ! fCO2 : One tracer transporting the total CO2 flux
          itc = itc + 1
          co2fields(it)%name='fCO2'
          co2fields(it)%id=it
          co2fields(it)%file='flx_fCO2.nc'
          IF (carbon_cycle_cpl) THEN 
             WRITE(lunout,*) 'carbon_cycle_mod (ESM) --- fCO2'
             co2fields(it)%cpl=.TRUE.
          ELSE
             co2fields(it)%cpl=.FALSE.
          END IF
          co2fields(it)%updatefreq = 86400*mth_len
          ! DOES THIS WORK ???? Problematic due to implementation of the coupled fluxes...
          CALL abort_physic('carbon_cycle_init','transport of total CO2 has to be implemented and tested',1)
       END SELECT
    END DO

    ! CO2 tracer

IF ( carbon_cycle_cpl .AND. carbon_cycle_tr) THEN
    itc=0

    DO it=1,nbtr
       iiq=niadv(it+nqo)

       SELECT CASE(tname(iiq))
       CASE("fCO2")
          itc = itc + 1
          co2fields(it)%name='fCO2'
          IF (carbon_cycle_cpl .AND. type_ocean=='couple') THEN
             co2fields(it)%cpl=.TRUE.
          END IF
        END SELECT
     END DO
END IF

!!!!! BIEN FAIRE LA DIFFERENCE ENTRE tracer.def et coupling_fields.def
! cela conditionne la valeur de ntr_co2 
!    ntr_co2 = itc 

    ntr_co2=0
    IF (carbon_cycle_tr .AND. (nbcf .GT. 0)) ntr_co2=1    

    ! Definition of control varaiables for the tracers
    DO it=1,ntr_co2
       aerosol(co2trac(it)%id) = .FALSE.
       radio(co2trac(it)%id)   = .FALSE.
    END DO
    
    ! Vector indicating which timestep to read for each tracer
    ! Always start read in the beginning of the file
    co2trac(:)%readstep = 0
   

! 3) Allocate variables
! ---------------------
    ! Allocate vector for storing fluxes to inject
!fCO2_nbp, fCO2_nep, fCO2_fLuc, fCO2_fFire, fCO2_fco2fos, fCO2_fgco2
    ALLOCATE(dtr_add(klon,maxco2fields), stat=ierr)
    IF (ierr /= 0) CALL abort_physic('carbon_cycle_init', 'pb in allocation 11',1)       

    ALLOCATE(dtr_add_sum(klon), stat=ierr)
    IF (ierr /= 0) CALL abort_physic('carbon_cycle_init', 'pb in allocation 12',1)   

 
    ! Allocate variables for cumulating fluxes from ORCHIDEE
!    IF (RCO2_inter) THEN
       !IF ((.NOT. carbon_cycle_tr) .AND. carbon_cycle_cpl) THEN
       IF (carbon_cycle_tr .OR. carbon_cycle_cpl) THEN

          ALLOCATE(fCO2_nbp(klon), stat=ierr)
          IF (ierr /= 0) CALL abort_physic('carbon_cycle_init', 'pb in allocation 2',1)
          fCO2_nbp(1:klon) = 0.

          ALLOCATE(fCO2_nep(klon), stat=ierr)
          IF (ierr /= 0) CALL abort_physic('carbon_cycle_init', 'pb in allocation 2',1)
          fCO2_nep(1:klon) = 0.
          
          ALLOCATE(fCO2_fLuc(klon), stat=ierr)
          IF (ierr /= 0) CALL abort_physic('carbon_cycle_init', 'pb in allocation 3',1)
          fCO2_fLuc(1:klon)   = 0.

          ALLOCATE(fCO2_fFire(klon), stat=ierr)
          IF (ierr /= 0) CALL abort_physic('carbon_cycle_init', 'pb in allocation 2',1)
          fCO2_fFire(1:klon) = 0.

       END IF
!    END IF


! 4) Test for compatibility
! -------------------------
!    IF (carbon_cycle_cpl .AND. type_ocean/='couple') THEN
!       WRITE(lunout,*) 'Coupling with ocean model is needed for carbon_cycle_cpl'
!       CALL abort_physic('carbon_cycle_init', 'coupled ocean is needed for carbon_cycle_cpl',1)
!    END IF
!
!    IF (carbon_cycle_cpl .AND..NOT. ok_veget) THEN
!       WRITE(lunout,*) 'Coupling with surface land model ORCHDIEE is needed for carbon_cycle_cpl'
!       CALL abort_physic('carbon_cycle_init', 'ok_veget is needed for carbon_cycle_cpl',1)
!    END IF

    ! Compiler test : following should never happen
    teststop=0
    DO it=1,teststop
       CALL abort_physic('carbon_cycle_init', 'Entering loop from 1 to 0',1)
    END DO


    IF (ntr_co2==0 .AND. carbon_cycle_tr) THEN
       ! No carbon tracers found in tracer.def. It is not possible to do carbon cycle 
! >> PC
       WRITE(lunout,*) 'No carbon tracers found in tracer.def. Not ok with carbon_cycle_tr and/or carbon_cycle_cpl'
! << PC
       CALL abort_physic('carbon_cycle_init', 'No carbon tracers found in tracer.def',1)
    END IF
    
! 5) Calculate total area of the earth surface
! --------------------------------------------
    CALL reduce_sum(SUM(cell_area),airetot)
    CALL bcast(airetot)

  END SUBROUTINE carbon_cycle_init


  SUBROUTINE carbon_cycle(nstep, pdtphys, pctsrf, tr_seri, source)
! Subroutine for injection of co2 in the tracers
!
! - Find out if it is time to update
! - Get tracer from coupled model or from file
! - Calculate new RCO2 value for the radiation scheme
! - Calculate CO2 flux to send to ocean and land models (PISCES and ORCHIDEE)

    USE infotrac_phy, ONLY: nbtr
    USE dimphy
    USE mod_phys_lmdz_transfert_para
    USE phys_cal_mod, ONLY : mth_cur, mth_len
    USE phys_cal_mod, ONLY : day_cur
    USE indice_sol_mod
    USE print_control_mod, ONLY: lunout
    USE geometry_mod, ONLY : cell_area
    USE infocfields_mod

! >> PC
! ne pas le mettre en argument de la subrourine
!    USE time_phylmdz_mod, ONLY: pdtphys
! << PC

    IMPLICIT NONE

    INCLUDE "clesphys.h"
    INCLUDE "YOMCST.h"

! In/Output arguments
    INTEGER,INTENT(IN) :: nstep      ! time step in physiq
    REAL,INTENT(IN)    :: pdtphys    ! length of time step in physiq (sec)
    REAL,DIMENSION(klon,nbsrf),INTENT(IN) :: pctsrf            ! Surface fraction
    REAL, DIMENSION(klon,klev,nbtr), INTENT(INOUT)  :: tr_seri ! All tracers
    REAL, DIMENSION(klon,nbtr), INTENT(INOUT)       :: source  ! Source for all tracers

! Local variables
    INTEGER :: it,itc
! >> PC
    LOGICAL :: newyear ! indicates if a new month just started
! << PC
    LOGICAL :: newmonth ! indicates if a new month just started
    LOGICAL :: newday   ! indicates if a new day just started
    LOGICAL :: endday   ! indicated if last time step in a day

! >> PC
    REAL, PARAMETER :: fact=1.0e06  * 28.97/44.011  ! transformation factor kg CO2/kg air => ppm
! << PC
    REAL, DIMENSION(klon) :: fco2_tmp
    REAL :: sumtmp
    REAL :: delta_co2_ppm
    

! 1) Calculate logicals indicating if it is a new month, new day or the last time step in a day (end day)
! -------------------------------------------------------------------------------------------------------

! >> PC 
! Mise a jour mensuelle des emissions fossiles 
    !newday = .FALSE.; endday = .FALSE.; newmonth = .FALSE.
    newday = .FALSE.; endday = .FALSE.; newmonth = .FALSE. ; newyear = .FALSE.

    IF (MOD(nstep,INT(86400./pdtphys))==1) newday=.TRUE.
    IF (MOD(nstep,INT(86400./pdtphys))==0) endday=.TRUE.
    IF (newday .AND. day_cur==1) newmonth=.TRUE.
! mettre la duree de l annee au propre len_year ???
    IF (newday .AND. (MOD(nstep,INT(86400.*365/pdtphys))==1)) newyear=.TRUE.
! << PC

! 2)  For each carbon tracer find out if it is time to inject (update)
! --------------------------------------------------------------------
    !DO it = 1, nbcf_in+2 
    DO it = 1, nbcf_in
       WRITE(lunout,*) 'nstep ',nstep
       WRITE(lunout,*) 'co2fields(it)%updatefreq ',co2fields(it)%name,' ', co2fields(it)%updatefreq
       WRITE(lunout,*) 'co2fields(it)%name ',co2fields(it)%name
       WRITE(lunout,*) 'INT(pdtphys) ',INT(pdtphys)
       WRITE(lunout,*) '(INT(co2fields(it)%updatefreq)/INT(pdtphys))', (INT(co2fields(it)%updatefreq)/INT(pdtphys))
       WRITE(lunout,*) 'MOD(nstep,(INT(co2fields(it)%updatefreq)/INT(pdtphys))) ',MOD(nstep,(INT(co2fields(it)%updatefreq)/INT(pdtphys)))
       IF ( MOD(nstep,(INT(co2fields(it)%updatefreq)/INT(pdtphys))) == 0 ) THEN
         co2fields(it)%updatenow = .TRUE.
       ELSE
         co2fields(it)%updatenow = .FALSE.
       END IF
       WRITE(lunout,*) 'co2fields(it)%updatenow ',co2fields(it)%updatenow
    END DO

! 3) Get tracer update
! --------------------------------------
! >> PC
       IF (newyear) THEN
         co2fields(:)%readstep = 0
       END IF
! << PC

    DO it = 1, nbcf_in 
       IF ( co2fields(it)%updatenow ) THEN
          IF ( co2fields(it)%cpl ) THEN
             ! Get tracer from coupled model
             SELECT CASE(TRIM(co2fields(it)%name))
             CASE('fCO2_nbp')     ! from ORCHIDEE
                dtr_add(:,it) = fCO2_nbp_inst(:)*pctsrf(:,is_ter)*((co2fields(it)%updatefreq)/INT(pdtphys))*fact !  [ppm/m2/s]
             CASE('fCO2_nep')     ! from ORCHIDEE
                dtr_add(:,it) = fCO2_nep_inst(:)*pctsrf(:,is_ter)*((co2fields(it)%updatefreq)/INT(pdtphys))*fact ! [ppm/m2/s]
             CASE('fCO2_fLuc') ! from ORCHIDEE
                dtr_add(:,it) = fCO2_fLuc_inst(:)  *pctsrf(:,is_ter)*((co2fields(it)%updatefreq)/INT(pdtphys))*fact ! [ppm/m2/s]
             CASE('fCO2_fFire')     ! from ORCHIDEE
                dtr_add(:,it) = fCO2_fFire_inst(:)*pctsrf(:,is_ter)*((co2fields(it)%updatefreq)/INT(pdtphys))*fact !  [ppm/m2/s]
             CASE('fCO2_fgco2')      ! from PISCES
                dtr_add(:,it) = fCO2_fgco2(:)  *pctsrf(:,is_oce)*((co2fields(it)%updatefreq)/INT(pdtphys))*fact ! [ppm/m2/s]
! >> PC
             CASE("fCO2_fco2fos")
               WRITE(lunout,*) 'carbon cycle --- fossil fuel emissions'
                 IF (newmonth) THEN
                   co2fields(it)%readstep = co2fields(it)%readstep + 1
! ne pas faire la lecture du fichier a chaque pas de temps
                   CALL read_map2D(co2fields(it)%file,'CO2_em_anthro',co2fields(it)%readstep,.TRUE.,source(1:klon,co2fields(it)%id))
                   dtr_add(:,it) = source(1:klon,co2fields(it)%id)*pctsrf(:,is_ter)*(co2fields(it)%updatefreq/INT(pdtphys))*fact ! [ppm/m2/s]
                 END IF
! << PC             
             CASE DEFAULT
                WRITE(lunout,*) 'Error with field ',co2fields(it)%name
                CALL abort_physic('carbon_cycle', 'No coupling implemented for this tracer',1)
             END SELECT
          ELSE
             ! Read tracer from file
             co2fields(it)%readstep = co2fields(it)%readstep + 1 ! increment time step in file
! Patricia   CALL read_map2D(co2fields(it)%file,'fco2',co2fields(it)%readstep,.FALSE.,dtr_add(:,it))
             CALL read_map2D(co2fields(it)%file,'fco2',co2fields(it)%readstep,.TRUE.,dtr_add(:,it))

! >> PC
             !! Converte from kgC/m2/h to kgC/m2/s
             !dtr_add(:,it) = dtr_add(:,it)/3600
! << PC
             ! Add individual treatment of values read from file
             SELECT CASE(co2fields(it)%name)
             CASE('fCO2_nbp')
                dtr_add(:,it) = dtr_add(:,it) *pctsrf(:,is_ter)
             CASE('fCO2_nep')
                dtr_add(:,it) = dtr_add(:,it) *pctsrf(:,is_ter)
             CASE('fCO2_fLuc')
                dtr_add(:,it) = dtr_add(:,it) *pctsrf(:,is_ter)
             CASE('fCO2_fFire')
                dtr_add(:,it) = dtr_add(:,it) *pctsrf(:,is_ter)
             CASE('fCO2_fgco2')
                dtr_add(:,it) = dtr_add(:,it) *pctsrf(:,is_oce)
             CASE('fCO2_fco2fos')
                dtr_add(:,it) = dtr_add(:,it) *pctsrf(:,is_ter)
! Patricia :
!             CASE('fCO2_fos_fuel')
!                dtr_add(:,it) = dtr_add(:,it)/mth_len
!                co2fields(it)%readstep = 0 ! Always read same value for fossil fuel(Cadule case)
             END SELECT
          END IF
       END IF
    END DO

! 4) Update co2 tracers : 
!    Loop over all carbon tracers and add source
! ------------------------------------------------------------------
    IF (carbon_cycle_tr) THEN
 
      dtr_add_sum(1:klon)=0.
      DO it=1,nbcf_in
         dtr_add_sum(1:klon)=dtr_add_sum(1:klon)+dtr_add(1:klon,it)
      END DO
 
       DO it = 1, ntr_co2
                 tr_seri(1:klon,1,co2trac(it)%id) = tr_seri(1:klon,1,co2trac(it)%id) + dtr_add_sum(1:klon)
       END DO
    END IF


! 5) Calculations for new CO2 value for the radiation scheme(instead of reading value from .def)
! ----------------------------------------------------------------------------------------------
    !IF (RCO2_inter) THEN
! >> PC
!       IF (.NOT. carbon_cycle_tr .AND. carbon_cycle_cpl) THEN
!!          IF (newday) THEN ! Reset cumulative variables once a day 
!!             fCO2_nep(1:klon) = 0.
!!             fCO2_fLuc(1:klon)   = 0.
!!          END IF
!! >> PC
!          fCO2_nep(1:klon) =  fCO2_nep_inst(1:klon) ![gC/m2/s]
!          fCO2_fLuc(1:klon)   =  fCO2_fLuc_inst(1:klon)   ![gC/m2/s]
!       END IF
!! << PC

! >> PC
    !   IF (endday) THEN
! << PC

! >> PC
! modifier le test (carbon_cycle_tr) ligne 492 pour prendre en compte que les cas
! ou carbon_cycle_cpl = 1 ou 2 .AND. carbon_cycle_tr=y
! bien declarer la variable fco2_tmp (pour un seul traceur)

          IF (carbon_cycle_tr) THEN
          WRITE(lunout,*) 'fco2_tmp --- carbon_cycle_tr case'
             ! Sum all co2 tracers to get the total delta CO2 flux
             fco2_tmp(:) = 0.
             DO it = 1, ntr_co2
                fco2_tmp(1:klon) = fco2_tmp(1:klon) + tr_seri(1:klon,1,co2trac(it)%id)
             END DO
! << PC             
          ELSE IF (carbon_cycle_cpl .AND. (.NOT. carbon_cycle_tr)) THEN ! no carbon_cycle_tr
          WRITE(lunout,*) 'fco2_tmp --- carbon_cycle_cpl case'
             ! Sum co2 fluxes comming from coupled models and parameter for fossil fuel
! >> PC
             !fco2_tmp(1:klon) = (fCO2_fco2fos*pctsrf(1:klon,is_ter) + (fCO2_fLuc(1:klon) + fCO2_nep(1:klon))*pctsrf(1:klon,is_ter) &

!             fco2_tmp(1:klon) = (fCO2_fco2fos(1:klon)*pctsrf(1:klon,is_ter) + fCO2_nbp(1:klon)*pctsrf(1:klon,is_ter) &
!                  + fCO2_fgco2(1:klon)*pctsrf(1:klon,is_oce)) * fact
! fCO2_fco2fos_1D
             fco2_tmp(:) = 0. 
             fco2_tmp(:) =  ( fCO2_fco2fos_1D + fCO2_nbp_inst(:)*pctsrf(:,is_ter) + fCO2_fgco2(:)*pctsrf(:,is_oce))/2.12 
             WRITE(lunout,*) 'fco2_tmp(:) ',fco2_tmp(:)
! << PC
          END IF

          ! Calculate a global mean value of delta CO2 flux
          fco2_tmp(:) = fco2_tmp(:) * cell_area(:)
          CALL reduce_sum(SUM(fco2_tmp),sumtmp)
          CALL bcast(sumtmp)
          delta_co2_ppm = sumtmp/airetot
          WRITE(lunout,*) 'delta_co2_ppm: ',delta_co2_ppm
          
          ! Add initial value for co2_ppm and delta value
           WRITE(lunout,*)'carbon_cycle_mod --- co2_ppm0 ',co2_ppm0
          co2_ppm = co2_ppm0 + delta_co2_ppm
          CALL bcast(co2_ppm)
          WRITE(lunout,*)'carbon_cycle_mod --- co2_ppm ',co2_ppm
IF (RCO2_inter) THEN          
          ! Transformation of atmospheric CO2 concentration for the radiation code
          RCO2 = co2_ppm * 1.0e-06  * 44.011/28.97 
          
          WRITE(lunout,*) 'RCO2 is now updated (in carbon_cycle_mod) ! RCO2 = ', RCO2

! >> PC
    !   END IF ! endday
! << PC

END IF ! RCO2_inter


! 6) Calculate CO2 flux to send to ocean and land models : PISCES and ORCHIDEE         
! ----------------------------------------------------------------------------
!    IF (carbon_cycle_cpl) THEN
!
!       IF (carbon_cycle_tr) THEN
!          ! Sum all co2 tracers to get the total delta CO2 flux at first model layer
!          fco2_tmp(:) = 0.
!          DO it = 1, ntr_co2
!             fco2_tmp(1:klon) = fco2_tmp(1:klon) + tr_seri(1:klon,1,co2fields(it)%id)          
!          END DO
!          co2_send(1:klon) = fco2_tmp(1:klon) + co2_ppm0
!       ELSE
!          ! Send a scalare value in 2D variable to ocean and land model (PISCES and ORCHIDEE)
!          co2_send(1:klon) = co2_ppm
!       END IF
!
!    END IF


! << PC
! co2_ppm0 au premier pas de temps
! prendre le co2 du restart

       IF (carbon_cycle_cpl .AND. carbon_cycle_tr) THEN
          ! Sum all co2 tracers to get the total delta CO2 flux at first model
          ! layer
          !fco2_tmp(:) = 0.
          DO it = 1, ntr_co2
             fco2_tmp(1:klon) = fco2_tmp(1:klon) + tr_seri(1:klon,1,co2fields(it)%id)
          END DO
          co2_send(1:klon) = fco2_tmp(1:klon) + co2_ppm0
       ELSE IF (carbon_cycle_cpl .AND. (.NOT. carbon_cycle_tr)) THEN
          WRITE(lunout,*) 'carbon_cycle_mod --- co2_send(1:klon)', co2_send(1:klon)
          co2_send(1:klon) = co2_ppm 
       ELSE
          ! Send a scalare value in 2D variable to ocean and land model (PISCES
          ! and ORCHIDEE)
          co2_send(1:klon) = co2_ppm
       END IF



  END SUBROUTINE carbon_cycle
  
END MODULE carbon_cycle_mod
